﻿using System;
using Luticate2.Auth.Business.Auth;
using Luticate2.Auth.Business.Auth.FieldsExpressions;
using Luticate2.Auth.Business.Auth.PartialObjectCopier;
using Luticate2.Auth.Business.FieldsExpressions;
using Luticate2.Auth.Business.FieldsExpressions.Basic;
//using Luticate2.Auth.Business.Pagination.OrderBy;
//using Luticate2.Auth.Business.Pagination.OrderBy.Basic;
using Luticate2.Auth.Business.PartialObjectCopier;
using Luticate2.Auth.ConsoleSample.Commands;
using Luticate2.Auth.DataAccess;
using Luticate2.Auth.DataAccess.Models;
using Luticate2.Auth.Dbo.Auth;
using Luticate2.Auth.Interfaces;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using NClap.Metadata;
using NClap.Repl;

namespace Luticate2.Auth.ConsoleSample
{
    enum CommandsEnum
    {
        [Command(typeof(ListCommand), Description = "Lists Luticate2 items", LongName = "ls", ShortName = "l")]
        ListItems,

        [Command(typeof(CreateCommand), Description = "Creates Luticate2 items", LongName = "create", ShortName = "c")]
        CreateItems,

        [Command(typeof(ExitCommand), Description = "Exits the shell")]
        Exit
    }
    
    class Program
    {
        public static IServiceProvider ServiceProvider;
        
        static void Main(string[] args)
        {
            IServiceCollection services = new ServiceCollection();
            
            services.AddScoped<LuGroupsBusiness>();

            services.AddSingleton<ILuPartialObjectCopier<LuObjectsMetadata, LuObjectsMetadataDbo>, LuPOCObjectsMetadataToDbo>();
            services.AddSingleton<ILuPartialObjectCopier<LuGroups, LuGroupDbo>, LuPOCGroupsToDbo>();
            services.AddSingleton<ILuPartialObjectCopier<LuGroupDbo, LuGroups>, LuPOCGroupsToModel>();


            services.AddSingleton<ILuFieldsExpressions<LuObjectsMetadataDbo, LuObjectsMetadata>, LuFieldsExpressionsLuMetadataDboLuMetadata>();
            services.AddSingleton<ILuFieldsExpressions<LuGroupDbo, LuGroups>, LuFieldsExpressionsLuGroupDboLuGroups>();

            services.AddSingleton<ILuFieldsExpressions<string, string>, LuFieldsExpressionsString>();
            services.AddSingleton<ILuFieldsExpressions<int, int>, LuFieldsExpressions<int, int>>();
            services.AddSingleton<ILuFieldsExpressions<Guid, Guid>, LuFieldsExpressions<Guid, Guid>>();

            services.AddSingleton<ILuFieldsExpressions<DateTime, DateTime>, LuFieldsExpressions<DateTime, DateTime>>();
            services.AddSingleton<ILuFieldsExpressions<DateTimeOffset, DateTimeOffset>, LuFieldsExpressions<DateTimeOffset, DateTimeOffset>>();
            services.AddSingleton<ILuFieldsExpressions<DateTime, DateTimeOffset>, LuFieldsExpressions<DateTime, DateTimeOffset>>();
            services.AddSingleton<ILuFieldsExpressions<DateTimeOffset, DateTime>, LuFieldsExpressions<DateTimeOffset, DateTime>>();

            services.AddSingleton<ILuFieldsExpressions<DateTime?, DateTime?>, LuFieldsExpressions<DateTime?, DateTime?>>();
            services.AddSingleton<ILuFieldsExpressions<DateTimeOffset?, DateTimeOffset?>, LuFieldsExpressions<DateTimeOffset?, DateTimeOffset?>>();
            services.AddSingleton<ILuFieldsExpressions<DateTime?, DateTimeOffset?>, LuFieldsExpressions<DateTime?, DateTimeOffset?>>();
            services.AddSingleton<ILuFieldsExpressions<DateTimeOffset?, DateTime?>, LuFieldsExpressions<DateTimeOffset?, DateTime?>>();

            services.AddDbContext<Luticate2DbContext>(options =>
            {
                options.UseNpgsql(@"Host=localhost;Database=luticate2;Username=dev;Password=dev");
                options.UseInternalServiceProvider(new ServiceCollection()
                    .AddEntityFrameworkNpgsql()
                    .BuildServiceProvider());
            }, ServiceLifetime.Transient);

            ServiceProvider = services.BuildServiceProvider();
            
            var loop = new Loop(typeof(CommandsEnum));
            loop.Execute();
        }
    }
}